<?php

namespace App\Datatables;

use Doctrine\ORM\Mapping\Entity;
use Sg\DatatablesBundle\Datatable\AbstractDatatable;
use Sg\DatatablesBundle\Datatable\Column\Column;
use Sg\DatatablesBundle\Datatable\Filter\TextFilter;
use Sg\DatatablesBundle\Datatable\Style;

/**
 * ClassBaseDatatable
 * @package App\Datatables
 */
abstract class BaseDatatable extends AbstractDatatable
{

    const ICON_DETAIL = 'fa fa-eye';
    const ICON_DELETE = 'fas fa-trash';
    const ICON_LIST = ' fas fa-th-list';
    const ICON_EDIT = ' fas fa-pen';
    const DATETIME_FORMAT = 'DD.MM.YYYY, HH:mm:ss';

    /**
     * @var string
     */
    protected string $entity;

    /**
     * @var string
     */
    protected string $name;

    /**
     * {@inheritdoc}
     */
    public function buildDatatable(array $options = array())
    {
        $this->language->set(array(
            'cdn_language_by_locale' => false,
        ));

        $this->ajax->set(array());

        $this->options->set(array(
            //'dom' => 'Bfrtip',
            'dom' => '<"top"Bfi>rt<"bottom"lp><"clear">',
            'classes' => Style::BOOTSTRAP_4_STYLE,
            'individual_filtering' => true,
            'individual_filtering_position' => 'head',
            'order_cells_top' => true,
            'length_menu' => [10, 25, 50, 100, 500],
            'page_length' => 50,
        ));

        $this->features->set(array(
            'processing' => true,
        ));

        $this->extensions->set(array(
            'select' => array(
                'blurable' => false,
                'class_name' => 'selected',
                'info' => true,
                'items' => 'row',
                'selector' => 'td, th',
                'style' => 'os',
            ),
            //'responsive' => true,
            'responsive' => array(
                //'details' => true,
                'details' => array(
                    'display' => array(
                        'template' => 'datatable/display.js.twig',
                        //'vars' => array('id' => '2', 'test' => 'new value'),
                    ),
                    'renderer' => array(
                        'template' => '/datatable/renderer.js.twig',
                    ),
                ),
            ),
        ));
    }

    /**
     * Get User.
     *
     * @return mixed|null
     */
    protected function getUser(){

        if ($this->authorizationChecker->isGranted('IS_AUTHENTICATED_FULLY')) {
            return $this->securityToken->getToken()->getUser();
        } else {
            return null;
        }

    }

    /**
     * Is admin.
     *
     * @return bool
     */
    protected function isAdmin(): bool {

        return $this->authorizationChecker->isGranted('ROLE_ADMIN');

    }

    /**
     * @return bool
     */
    protected function isSuperAdmin(): bool {

        return $this->authorizationChecker->isGranted('ROLE_SUPER_ADMIN');

    }

    protected function addIdColumn(){

        $this->columnBuilder->add('id', Column::class, array(
            'title' => 'Id',
            'width' => '100%;',
            'class_name' => 'row_id',
            'searchable' => false,
            'filter' => array(TextFilter::class,
                array(
                    'cancel_button' => true,
                ),
            ),
        ));

    }

    /**
     * @return array
     */
    protected function getRefreshButton(){

        return array(
            'action' => array(
                'template' => '/datatable/button/refresh.js.twig',
                'vars' => array('tableName' => 'sg-datatables-' . $this->getName()),

            ),
            'class_name' => 'refresh',
            'name' => 'refresh',
            'text' => '<i class=\"fas fa-sync\"></i> Refresh',
            'title_attr' => 'Refresh',
        );

    }

    /**
     * @return array
     */
    protected function getAutoRefreshButton(){

        return array(
            'action' => array(
                'template' => '/datatable/button/autoRefresh.js.twig',
                'vars' => array('tableName' => 'sg-datatables-' . $this->getName()),

            ),
            'class_name' => 'auto-refresh',
            'name' => 'auto-refresh',
            'text' => '<i class=\"fas fa-sync\"></i> Auto Refresh',
            'title_attr' => 'Auto Refresh',
        );

    }

    /**
     * @return string
     */
    public function getEntity(){

        return $this->entity;

    }

    /**
     * @return string
     */
    public function getName(){

        return $this->name;

    }

    /**
     * @param array $array
     * @param string $keyName
     * @param string $valueName
     * @return array
     */
    protected function getOptionsFromArray(array $array, string $keyName, string $valueName){

        $options = [];

        foreach ($array as $item){

            $options[$item[$keyName]] = $item[$valueName];
        }

        return $options;

    }

}